	list	p=16f877A	; list directive to define processor
	include	<p16F877A.inc>
    
	errorlevel -302,-207	; suppress message 302 from list file
	errorlevel -305,-207	; suppress message 302 from list file

	__config _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _XT_OSC & _WRT_OFF & _LVP_OFF & _CPD_OFF

;===========================================================================  
;
;     PIC16F877-A   
;                                 __________  
;                 +5v------!MCLR |1       40| RB7-I---- (KEYPAD  COL4)  
;            (Unused)------  RA0 |2       39| RB6-I---- (KEYPAD  COL3)
;            (Unused)------  RA1 |3       38| RB5-I---- (KEYPAD  COL2)
;            (Unused)------  RA2 |4       37| RB4-I---- (KEYPAD  COL1)
;            (Unused)------  RA3 |5       36| RB3  ---- (Unused)  
;            (Unused)------  RA4 |6       35| RB2  ---- (Unused)  
;            (Unused)------  RA5 |7       34| RB1  ---- (Unused)                  
;            (Unused)------  RE0 |8       33| RB0/INT-- (Unused)  
;            (Unused)------  RE1 |9       32| VDD  ---- +5v  
;            (Unused)------  RE2 |10      31| VSS  ---- GND  
;                 +5v------  VDD |11      30| RD7-O---- (KEYPAD  ROW4)   
;                 GND------  VSS |12      29| RD6-O---- (KEYPAD  ROW3)
;                XTAL------ OSC1 |13      28| RD5-O---- (KEYPAD  ROW2)  
;                XTAL------ OSC2 |14      27| RD4-O---- (KEYPAD  ROW1) 
;             (Unused)-----  RC0 |15      26| RC7-O---- (LCD pin 14)  LCD_DB7    
; LCD_rs (LCD pin 4) ------O-RC1 |16      25| RC6-O---- (LCD pin 13)  LCD_DB6  
; LCD_rw (LCD pin 5) ------O-RC2 |17      24| RC5-O---- (LCD pin 12)  LCD_DB5  
; LCD_e  (LCD pin 6) ------O-RC3 |18      23| RC4-O---- (LCD pin 11)  LCD_DB4  
;            (Unused)------  RD0 |19      22| RD3  ---- (Unused)  
;            (Unused)------  RD1 |20      21| RD2  ---- (Unused)  
;                                 ----------  
;
;	LCD (Hitachi)  
;                                 ________  
;    		                 VSS |1       |   
;                            VDD |2       |   
;                 CONTRAST   VEE |3       |   
;                 RC1------  RS  |4       |   
;                 RC2------  R/W |5       |   
;                 RC3------  E   |6       |   
;                       (Unused) |7       |                   
;                       (Unused) |8       |   
;                       (Unused) |9       |   
;                       (Unused) |10      |  
;                 RC4------  DB4 |11      |    
;                 RC5------  DB5 |12      |   
;                 RC6------  DB6 |13      |      
;                 RC7------  DB7 |14      |     
;          BACKLIGHT POWER   VDD |15      |     
;                   GROUND   VSS |16      |   
;                                 --------  
;
;
;    KEYPAD
;                     RB4        RB5         RB6          RB7
;                      |          |           |            |
;                      |          |           |            |
;                      |          |           |            |
;
;                    COL 1       COL 2       COL 3        COL 4   
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 1 |    1    |  |   2    |  |   3    |   |   A    |
;                 +---------+  +--------+  +--------+   +--------+ 
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 2 |    4    |  |   5    |  |   6    |   |   B    |
;                 +---------+  +--------+  +--------+   +--------+ 
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 3 |    7    |  |   8    |  |   9    |   |   C    |
;                 +---------+  +--------+  +--------+   +--------+  
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 4 |    *    |  |   0    |  |   #    |   |   D    |
;                 +---------+  +--------+  +--------+   +--------+  
;
;
;=============================================================================
; DECLARATIONS
;=============================================================================
RESET_V		equ	0X0000			; ADDRESS OF RESET VECTOR.
ISR_V		equ	0X0004			; ADDRESS OF INTERRUPT VECTOR.
OSC_FREQ 	equ D'4000000'		; OSCILLATOR FREQUENCY IS 4 MHZ.

LCD_RS		equ	1				;0 = COMMANDS. 1 = DATA.
LCD_RW		equ	2				;0 = WRITE TO LCD. 1 = READ LCD.
LCD_E		equ	3				;ENABLE THE LCD TO ACCEPT INPUT. PULSE THIS.
LCDBUSY		equ	7				;WHEN THE LCD IS BUSY RC7 IS SET HIGH.


; PORTB BITS
RB7			equ	7				;KEYBOARD, COL 4.
RB6			equ	6				;KEYBOARD, COL 3.
RB5			equ	5 				;KEYBOARD, COL 2.
RB4			equ	4				;KEYBOARD, COL 1.

; PORTD BITS
RD7			equ	7				;KEYBOARD, ROW 1.
RD6			equ	6				;KEYBOARD, ROW 2.
RD5			equ	5				;KEYBOARD, ROW 3.
RD4			equ	4				;KEYBOARD, ROW 4.

RD3			equ	3				;KEY CODE, BIT 3, USE 1K + LED TO VSS.
RD2			equ	2				;KEY CODE, BIT 2, USE 1K + LED TO VSS.
RD1			equ	1				;KEY CODE, BIT 1, USE 1K + LED TO VSS.
RD0			equ	0				;KEY CODE, BIT 0, USE 1K + LED TO VSS.

;INTERRUPT BIT SETTINGS.
ENC_UP_BIT	equ	1
ENC_DN_BIT 	equ 2
KEYS_BIT	equ	0		

;OTHER BIT SETTINGS.
MENU_BIT	equ 0				;BIT FOR USER BEING IN MENU.
INT_BIT		equ 0
		

; REGISTERS.
		cblock  H'20'       ; NOTE: START DATA BLOCK.
;FREQUENCY "ENTRY" VALUES.
POS_10M	
POS_1M
POS_100K
POS_10K
POS_1K
POS_100
POS_10
POS_1						;** FCHAR_STRT MUST BE SET TO THIS ADDRESS **

FREQ_10M	
FREQ_1M
FREQ_100K
FREQ_10K
FREQ_1K
FREQ_100
FREQ_10
FREQ_1						;** FCHAR_FREQ MUST BE SET TO THIS ADDRESS **

FCHAR_MAX					;THIS IS THE TRUE COUNT OF CHARS ENTERED.
FCHAR_MAX_TEMP				;THIS IS THE TRUE COUNT OF OLD FREQ ENTERED.
FCHAR_CNT					;USED EVERYTIME WE SHIFT CHARS (UPDATED).
FCHAR_CNT1					;USED WHEN TRANSFERING FROM POS_XX TO FREQ_XX
FCHAR_STRT					;START OF POS_1 IN ADDRESS LOCATION...
FCHAR_TEMP					;START OF TEMP_1 IN ADDRESS LOCATION...
FCHAR_FREQ					;START OF FREQ_1 IN ADDRESS LOCATION...

POS1
POS2

;ENCODER STEP VARIABLE.
STEP_VAL

;EEPROM VARIABLES.
ADDR
VALUE

;KEYPAD VARIABLES.       		
TEMP						;TEMP STORAGE.
LAST_KEY					;LAST KEY PRESSED.
LAST_CHAR					;LAST KEY PRESSED.

		
TIMER1						; USED IN DELAY ROUTINES
TIMER2						;   " 

;LCD VARIABLES.
LCD_CHAR					;THE BYTE REPRESENTING THE LCD CHARACTER. 
LCD_READ 					;VARIABLE CONTAINING THE READ FOR LCD_BUSY. 
LCD_VALUE 					;THE LCD RS LINE - FLAG VALUE. 

;INTERRUPT FLAG VARIABLE.
INTERRUPT_FLAG

;OTHER FLAG VARIABLES.
EEPROM_FLAG
NEW_FREQ_FLAG
MENU_FLAG

MENU_COUNT					;USED TO INCREMENT UPON DISPLAYING NEW MENU ITEM.
		
;INTERRUPT TEMPORARY SWAP OUT VARIABLES.
W_TEMP
STATUS_TEMP
PCLATH_TEMP

;DELAY VARIABLES.
DELAY_VAL					; USED IN DELAY ROUTINES.
X_DLY_VAL					; "
COUNT						; USED AS G.P. COUNTER.

		endc				; NOTE: END OF DATA BLOCK.    
   
;=============================================================================
; PROGRAM START.
;***************
;
; NOTE: - 	Programs always start at ORG 0x00. The reset vector always starts
;			at 0x04 so four lines down from 00. This means if you are 
;			including an interupt routine you must jump over 0x04 or your 
;			will run into it.		
;=============================================================================
		org		RESET_V		; RESET vector location
		reset	GOTO MAIN	;JUMPING TO AVOID THE INTERRUPT VECTOR.

;=============================================================================
;THIS IS THE INTERUPT ROUTINE.
;
;PURPOSE: - To interrupt the microcontroller inorder to perform the following 
;			tasks:
;				- what interrupt was it??
;				- Set the approriate flag.
;			Its important to get in and get out of the interrupt routine
;			effectively... so we set flags rather than do any work.
;
;CODE 
;SOURCE: - 	Much of this routine was recovered from MICROCHIP'S DATASHEET. See
;			page ?? where it has examples of the INTERRUPT routine for and
;			how to save off the special registers and then after recover them. 
;=============================================================================
INTERRUPT	org	ISR_V		;INTERRUPT VECTOR LOCATION.

		movwf	W_TEMP 		;COPY W TO TEMP REGISTER
		swapf	STATUS,	W 	;SWAP STATUS TO BE SAVED INTO W
		clrf	STATUS 		;BANK 0, REGARDLESS OF CURRENT BANK, CLEARS IRP,RP1,RP0
		movwf	STATUS_TEMP ;SAVE STATUS TO BANK ZERO STATUS_TEMP REGISTER
		movf	PCLATH, W 	;ONLY REQUIRED IF USING PAGES 1, 2 AND/OR 3
		movwf	PCLATH_TEMP ;SAVE PCLATH INTO W
		clrf	PCLATH 		;PAGE ZERO, REGARDLESS OF CURRENT PAGE



		btfsc	INTCON,	INTF;INTERRUPT FROM ENCODER.
		goto	ENCODER_INT
		goto 	KEYS_INT

ENCODER_INT
		call 	FLASH
		goto	DONE




KEYS_INT
		call	WAIT_15MS

		movlw   B'11110000'	;CHECK FOR KEY DOWN.
		movwf   TEMP  
		movf    PORTB,	W
		subwf   TEMP,	F
		btfsc	STATUS, Z
		goto    DONE		;ALL KEYS UP SO RETURN.

		call	GETKEY		;GETKEY IS IN THE KEYPAD INCLUDE FILE.
		movwf	LAST_CHAR

		bsf		INTERRUPT_FLAG,	KEYS_BIT
		
		clrf    PORTD		;CLEAR PORT D.

DONE
		bcf     INTCON,RBIF;CLEAR Port B interrupt FLAG
		bcf     INTCON,INTF;CLEAR Port B interrupt FLAG

		movf	PCLATH_TEMP, W ;RESTORE PCLATH
		movwf	PCLATH 		;MOVE W INTO PCLATH
		swapf	STATUS_TEMP,W ;SWAP STATUS_TEMP REGISTER INTO W
							;(SETS BANK TO ORIGINAL STATE)
		movwf	STATUS 		;MOVE W INTO STATUS REGISTER
		swapf 	W_TEMP,F 	;SWAP W_TEMP
		swapf 	W_TEMP,W 	;Swap W_TEMP into W

		retfie

;=============================================================================
;INCLUDE FILES HERE.
;*******************
;=============================================================================
		include "KEYPAD_ROUTINES.INC"
		include "LCD_ROUTINES.INC"
		include "LCD_LABELS.INC"

;=============================================================================
;"MAIN" STARTS HERE.
;*******************
;
;PURPOSE: - Here we call the PIC initialisation routine and the LCD 
;			initialisation routine... display an opeing message, we set our
;			pivotal variables and call our start routine.
;=============================================================================	
MAIN
		call	WAIT_15MS	;GIVE THE MICRO A CHANCE TO SETTLE DOWN.
		call	PIC_INIT	;INITIATE THE PIC - SET PORTS ENABLE WEAK PULLUPS.
		call	LCD_INIT	;INITIATE THE LCD NEXT (LCD_ROUTINES.INC).
		call	FLASH		;FLASH ALL FOUR LEDS MOMENTARILY.
		call	OPEN_MESG	;DISPLAY OPENING MESSAGE.

							;NOTE - We construct our frequency using eight
							; bytes. The first is situated at the address
							; H'27' and goes UP ie singles at H'27' tens
							; at H'26' and so on. We need to set our 
							; variables first and  this includes the first
							; just mentioned - H'27' for our location just
							; temporary location - H'37' for our frequency 
							; location.

							; FCHAR_MAX is PIVOTAL in this program.
							; It tells us how many numbers have been 
							; entered.
						
		movlw	H'27'
		movwf	FCHAR_STRT	;SO SET OUR FIRST "POS" OR TEMPORARY LOCATION.

		movlw	H'2F'
		movwf	FCHAR_FREQ	;NOW SET OUR "FREQ" LOCATION.

		movlw	H'00'
		movwf	FCHAR_MAX	;ALWAYS START AT "0" FOR OUR ENTER COUNT.
		
		movlw	H'00'
		movwf	INTERRUPT_FLAG ; OUR INTERRUPT FLAG MUST BE CLEARED.

		movlw	H'00'
		movwf	NEW_FREQ_FLAG ; NOW CLEAR OUR FREQUENCY ENTER FLAG.

		movlw	H'00'
		movwf	MENU_FLAG 	; NOW CLEAR OUR FREQUENCY ENTER FLAG.

		bcf		NEW_FREQ_FLAG,	INT_BIT	; CLEAR OUR EEPROM FLAG.

							;NOTE: - This next routine goes off first to
							; read the eerpom, check if it has anything
							; in it, then based on if it has or hasn't
							; set the EEPROM_FLAG accordingly. By the way
							; if the eeprom has a frequency it loads
							; this into the FCHAR_FREQ registers ready
							; for the LCD to display them...

		call 	GET_STARTUP_FREQ ; OUR FIRST ROUTINE. 

							;NOTE: - having set the EERPOM_FLAG it now
							; checks it. If it has been set then we aleady 
							; have the values in the  eeprom loaded into 
							; the FCHAR_FREQ and so instead of just going
							; to LOOP and waiting for an entry we can
							; simply run the routine that displays this 
							; frequency on the LCD.

		btfsc	EEPROM_FLAG,0 ; DOES THE EEPROM CONTAIN A FREQUENCY?
		goto	LOOP		;NO.. ITS NOT SET SO JUST WAIT FOR THE 
							;USER TO ENTER ONE IN...
		call	DISPLAY_FREQ_LCD ; YES. NOTHING WAS SET SO DISPLAY THE 
							; FREQUENCY THAT HAS BEEN LOADED INTO FCHAR_FREQ.
		goto 	LOOP		;...AND GO AND WAIT AT LOOP.

;=============================================================================
;THE MAIN LOOP USED FOR THE INTERRUPTS.
;
;PURPOSE: - Its here that the micro just loops over and over again until a
;			key is pressed or the encoder has been turned.
;=============================================================================		
LOOP
		btfsc	INTERRUPT_FLAG,KEYS_BIT	; INTERRUPT FLAG BEEN SET?
		call	KEY_INTERRUPT	;YES. SO GO FIND OUT WHAT KEY AND PROCESS....
		
		goto	LOOP		;LOOP BACK.


;WORKING SPACE**************************************************************
;***************************************************************************


MAIN1
		call	PIC_INIT	;INITIATE THE PIC.
		movlw	H'27'
		movwf	FCHAR_STRT

		movlw	H'2F'
		movwf	FCHAR_FREQ

		movlw	H'00'
		movwf	FCHAR_MAX


		call	TEST1
		call	LOAD_NEW_FRQ

		goto 	LOOP



EEPROM_START_LOAD1	
		movlw	H'09'
		movwf	VALUE

		movlw	H'07'
		movwf	ADDR

		call WRITE_EEPROM

		movlw	H'04'
		movwf	VALUE

		movlw	H'06'
		movwf	ADDR

		call WRITE_EEPROM

		movlw	H'03'
		movwf	VALUE

		movlw	H'05'
		movwf	ADDR

		call WRITE_EEPROM
		return

		call 	GET_STARTUP_FREQ

		goto	LOOP


;****************************





TEST1

		bcf 	STATUS, RP1 ;BANK 0
		bcf 	STATUS, RP0 ;
		movlw	H'06'
		movwf	POS_1

		movlw	H'06'
		movwf	POS_10
		
		movlw	H'06'
		movwf	POS_100

		movlw	H'07'
		movwf	POS_1K

		movlw	H'09'
		movwf	POS_10K

		movlw	H'06'
		movwf	POS_100K

		movlw	H'00'
		movwf	POS_1M

		movlw	H'07'
		movwf	FCHAR_MAX	
		
		movlw	H'27'
		movwf	FCHAR_STRT
		bsf		EEPROM_FLAG,0 ;THIS TELLS THE SYSTEM TO SAVE TO EEPROM.		
		;call 	LOAD_NEW_FRQ

		return

	

	



		
	







DEBUG
		call	LCD_LINE1
	MOVF FCHAR_FREQ,W
		call 	LCD_PUT_ASC
	MOVF FREQ_1,W
			call 	LCD_PUT_ASC
		RETURN

DEBUG_MEN
		call	LCD_LINE1
	MOVF MENU_FLAG,W
		call 	LCD_PUT_ASC

		RETURN

DEBUG_VALUE
		call	LCD_LINE1
	MOVF VALUE,W
		call 	LCD_PUT_ASC

		RETURN

DEBUGQ
		call	LCD_LINE1
		movlw	'Q'
		call 	LCD_DISP_CHAR
		
		RETURN

DEBUGS
		call	LCD_LINE1
		movlw	'S'
		call 	LCD_DISP_CHAR
		
		RETURN

DEBUGP
		call	LCD_LINE1
		movlw	'P'
		call 	LCD_DISP_CHAR
		
		RETURN


DEBUG_INT
		call	LCD_LINE1
		movF	NEW_FREQ_FLAG,W
		call 	LCD_PUT_ASC
		
		RETURN



DEBUG_
		call	LCD_LINE1
		movlw	' '
		call 	LCD_DISP_CHAR
		
		RETURN

DEBUG_MAX
		call	LCD_LINE1
		movf	FCHAR_MAX,W
		call 	LCD_PUT_ASC
		
		RETURN

DEBUG_POS_1
		call	LCD_LINE1
		movf	POS_1,W
		call 	LCD_PUT_ASC
		
		RETURN

DEBUGA
		call	LCD_LINE1
		movlw	'A'
		call 	LCD_DISP_CHAR
		
		RETURN

DEBUG_CHAR
		call	LCD_LINE1
		movf	LAST_CHAR,W
		call 	LCD_PUT_ASC
		
		RETURN
;=============================================================================
;INITIATE THE PIC MICROCONTROLLER.
;
;PURPOSE: - This routine intialises the micro. It clears the special purpose
;			registers, clears the ports, goes to BANK 1 so as to set half of
;			PORTB as INPUT. It also enables weak pullups on PORTB for the 
;			keypad peripheral...
;=============================================================================		
PIC_INIT
		clrf	STATUS		; DO INITIALIZATION, SELECT BANK 0
		clrf	INTCON		; CLEAR INT-FLAGS, DISABLE INTERRUPTS
		clrf	PCLATH		; KEEP IN LOWER 2KBYTE
	
		clrf	PORTA		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
		clrf	PORTB		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
		clrf	PORTC		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
		clrf	PORTD		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
		clrf	PORTE		; ALL PORT OUTPUT SHOULD OUTPUT LOW.
	
		bcf		STATUS, RP1	; FOR BANK 1 RP1=0 RP0=1.
		bsf		STATUS, RP0	; SELECT BANK 1.
	
		clrf    PORTB		;LATCH ALL LOW.
		movlw   B'11110011'	;RB4-RB7 INPUTS. 
		movwf	TRISB	
			
		movlw	B'00000000'	; RD4-7 INPUTS.
		movwf	TRISD
		
		bcf		OPTION_REG, NOT_RBPU	
							;ENABLE WEAK PULL-UPS.
							;ON PORT B.
	
		bcf		STATUS, RP1	; FOR BANK 0 RP1=0 RP0=0.
		bcf		STATUS, RP0	; SELECT BANK 0.

		bcf		STATUS,IRP	;MAKING SURE THAT THE FIRST PAGE OF THE FSR
							; IS SELECTED.
		
		return

;=============================================================================
;FLASH ALL FOUR LEDS.
;=============================================================================
FLASH		
		movlw	B'00001111'	;RD4-7 HAVE LEDS ON THEM (DEV BOARD) SET THESE
							;PORTS HIGH.
		movwf	PORTD		;NOW SEND THIS TO THE ACTUAL PORT
		call	WAIT_15MS	;FLASH FOR 15MS.
		movlw	B'00000000'	;NOW TURN THEM OFF.
		movwf	PORTD		;SEND THIS TO THE ACTUAL PORT AGAIN.
		
		return

;=============================================================================
;1. ROUTINES FOR STARTUP FREQUENCY TEST AND POS TO FREQ CONVERSION...
;*******************************************************************
;
;PURPOSE: - There are five majour routines in this block.
;
;1.1 GET_STARTUP_FREQ
;1.2 EE_VALIDATION
;1.3 EE_FREQ_DOESNT_EXIST
;1.4 EE_FREQ_EXISTS
;1.5 READ_EPROM 
;1.6 WRITE_EEPROM
;1.7 EEPROM_START_LOAD
;
;PURPOSE: - This routine works in unison with the next three routines.

;				1.2 EE_VALIDATION
;				1.3 EE_FREQ_DOESNT_EXIST
;				1.4 EE_FREQ_EXISTS
;
;			When the user turns on the function generator the systems checks
;			whether there is a start frequency entered. To do this it first
;			must check if there is anything located in the EEPROM. It does 
;			this by reading the first eight addresses in the EEPROM.
;			It then checks whether any of these addresses have anything in
;			them. It does this, by the way, by starting with the highest 
;			address first and working backwards thereby determining how many 
;			chars make up the starting frequency value. (loads this number into 
;			FCHAR_MAX - this is used to determine if there were any chars found).;
;
;			It goes off to "EE_VALIDATION" to deterime which routine to use...
;			if there are no numbers in the EERPOM it is sent to 
;			"EE_FREQ_DOESNT_EXIST". Here the interrupts are set and the 
;			EEPROM_FLAG is set. Further the LCD is set to display to the user
;			there is no starting frequency and that one needs to be entered.
;
;			If a value was found in the EEPROM then "EE_VALIDATION" directs
;			the program to goto "EE_FREQ_EXISTS" where it sets ALL the 
;			interrupts being used in the system including the encoder.			

;1.1 GET THE STARTUP FREQUENCY IN THE EEPROM.
;
;PURPOSE: - To cycle through the first eight locations in the EEPROM backwards
;			from "07" to "00" checking if there is a value in that position and 
;			if so to save it using the FSR register to the FREQ positions. Also
;			to set FCHAR_MAX to "00" and then increment it everytime a
;			character is found in the EEPROM. 
;=============================================================================
GET_STARTUP_FREQ
		movf	FCHAR_FREQ,W;SET THE INDIRECT REGISTER TO THE LOWEST POS POSIT.
		movwf	FSR

		movlw	H'07'		;0 TO 7 -> 8 POSTIONS TO DECREMENT.
		movwf	ADDR		;SET ADDRESS TO DECREMENT 8 POSTIONS DOWN TO "0".

READ_NEXT_LOC
		btfsc	STATUS,Z	;IS "W" AT ZERO?. 	
		goto	EE_VALIDATION ; YES. SO NOW CHECK IF FCHAR_MAX HAS ANYTHING..

		movf	ADDR,W		;MOVE OUR POSTION TO "W"
		call	READ_EEPROM	;READ THE EEPROM AT THIS POSTION AND PUT IN "W".

		movwf	VALUE		;NOW PUT WHATS IN "W" INTO "VALUE".

							;NOTE: - Every empty position in the 64 byte
							; EEPROM has "FF" in it... so by subtracting "FF"
							; from it and checking the STATUS register Z to
							; to see if its set validates if anything is 
							; in the EEPROM.

		sublw	H'FF'
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	NO_VALUE	;YES. SO JUST DECREMENT BOTH VARIABLES USED.

		movf	VALUE,W		;NO. SO LETS SAVE THIS VALUE TO THE CURRENT FSR.
		movwf	INDF		;INDF IS THE REGISTER LOCATION THE FSR IS
							; POINTING TO.
		incf	FCHAR_MAX	;WE HAVE A VALUE!  THAT IS ITS > "0". THIS
							; TELLS US LATER WE HAVE A FREQ IN THE EEPROM.
		decf	FSR			;GOTO THE NEXT HIGHEST LOCATION (UP ONE REG).
		decf	ADDR		;DOWN ONE TO THE NEXT TEST FOR A CHAR.
		goto	READ_NEXT_LOC ; OK BACK UP TO DO IT ALL AGAIN.

NO_VALUE
		movlw	H'00'		;SET THAT LOCATION TO "0".
		movwf	INDF
		
		decf	FSR			;DECREMENT BY ONE THE INDIRECT REGISTER AND THE
		decf	ADDR		; "ADDR" REGISTER.

		goto	READ_NEXT_LOC ; GO BACK UP TO THE TOP TO DO IT ALL AGAIN.

;=============================================================================
;1.2 VALIDATES IF AN EPPROM VALUE EXISTS AND IF SO DIRECTS THE PROCESS TO
;	 DISPLAYING IT.
;
;PURPOSE: - After the system runs through all 7 locations in the eeprom and 
;			upon finding a frequency loads it into our frequency registers
;			it then is directed to this routine. This routine checks the 
;			character count as this was incremented everytime a character was
;			found... if its greater than "0" then we have a frequency and it 
;			is directed to go off and display that frequency.
;=============================================================================
EE_VALIDATION
		movf	FCHAR_MAX,F
		btfsc	STATUS,Z	;SKIP IF NOT ZERO...
		goto	EE_FREQ_DOESNT_EXIST
		goto	EE_FREQ_EXISTS

;=============================================================================
;1.3 FREQUENCY DOESNT EXIST
;
;PURPOSE: - By determining in "EE_VALIDATION" that FCHAR_MAX equals zero
;			we are directed to this routine. Here we set the interrupts for
;			just the keyboard and not the encoder as we need a frequency first.
;			We also set up the LCD to display the labels telling the user 
;			to enter a frequency and then we set the LCD to display the
;			enter mode.
;=============================================================================
EE_FREQ_DOESNT_EXIST
							;NOTE: - We have only enabled RB4-7 interrupts 
							; or the keypad here - NOT THE ENCODER INTERRRUPT.

		bsf		INTCON,RBIE	;ENABLE INTERRUPT ON RB4-7 CHANGE.
		bsf		INTCON,GIE	;ENABLE INTERRUPTS. 

		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call	NEW_STRT_FREQ_LABEL ; LABEL TO TELL USER TO ENTER FREQ.

		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call	LCD_CLEAR	;NOW CLEAR THE LCD COMPLETELY.


		bsf		EEPROM_FLAG,0 ;THIS TELLS THE SYSTEM TO SAVE TO EEPROM.

		call	LOAD_FREQ_LCD ; SETS UP THE LCD TO DISPLAY A LOAD PROMPT AND
							;	THE "Hz" LABEL.

		return				;(RETURN HERE)

;=============================================================================
;1.3 FREQUENCY EXISTS
;
;PURPOSE: - We have determined from above that a frequency exists so all we
;			need to do is set all the interrupts and make sure the EEPROM_FLAG
;			is cleared. We also need to clear the new frequency flag so the
;			system doesnt think its in the middle of entering in a frequency.
;			By returning all the system will do next is loop in LOOP until
; 			there is a keypad entry or encoder movement.
;=============================================================================	
EE_FREQ_EXISTS
		bsf		INTCON,RBIE	;ENABLE INTERRUPT ON RB4-7 CHANGE.
		bsf		INTCON,INTE	;ENABLE INTERRUPT ON RB0/INT CHANGE. 
		bsf		INTCON,GIE	; ENABLE INTERRUPTS. 

		bcf		INTCON,INTF	;CLEAR INTERRUPT FLAG.

		bcf		EEPROM_FLAG,0 ;THIS TELLS THE SYSTEM NOT TO SAVE TO EEPROM.

		bcf		NEW_FREQ_FLAG,INT_BIT ; ...AND WE ARE NOT IN THE MIDDLE OF
							;			ENTERING IN A NEW FREQUENCY.

		return				;(RETURN HERE)

;=============================================================================
;1.3 READS THE CONTENTS OF AN ADDRESS IN THE EEPROM.
;
;PURPOSE: - Reads the location in "W" as EEADR. Place the address location
;			into "W" and then call this routine. It will move to BANK 3, set
;			EEPGD in EECON1 and then by setting RD in EECON1 automatically
;			read the contents of that register.
;=============================================================================
READ_EEPROM      
		bsf 	STATUS, RP1	;GOTO BANK 2 RP1, RP0 -> "01".
		bcf 	STATUS, RP0	

		movwf 	EEADR		;MOVE WHATS INTO "W" INTO "EEADR".

		bsf 	STATUS, RP0	;GOTO BANK 3  RP1, RP0 -> "11"

		bcf 	EECON1, EEPGD ; POINT TO THE DATA MEMORY.
		bsf 	EECON1, RD	;NOW START READ OPERATION.

		bcf 	STATUS, RP0	;GOTO BANK 2 RP1, RP0 -> "01".

		movf 	EEDATA, W	;MOVE THE CONTENTS FROM "EEDATA" TO "W".

		bcf 	STATUS, RP1	;GOTO BANK 0 GOTO BANK 2 RP1, RP0 -> "00".

      	return

;=============================================================================
;1.4 WRITES THE CONTENTS SET IN "VALUE" INTO THE ADDRESS "ADDR" .
;
;PURPOSE: - "VALUE" and "ADDR" are set before calling this routine. It will
;			place the value in the variable "VALUE" into the EEPROM at address
;			"ADDR".
;=============================================================================
WRITE_EEPROM	
		bsf 	STATUS, RP1 ;GOTO BANK 3  RP1, RP0 -> "11"
		bsf 	STATUS, RP0 ;

		btfsc 	EECON1, WR 	;WAIT FOR WRITE TO FINISH.
		goto 	$-1 		;GO BACK UP ONE LINE IN OUR PROGRAM.

	
		bcf 	STATUS, RP1 ;GOTO BANK 0 GOTO BANK 2 RP1, RP0 -> "00".
		bcf 	STATUS, RP0 ;

		movf 	ADDR, W 	;MOVE OUR ADDRESS (ALREADY SET) TO "W".

		bsf 	STATUS, RP1 ;GOTO BANK 2 RP1, RP0 -> "01".

		movwf 	EEADR 		;MOVE WHATS INTO "W" INTO "EEADR".

		bcf 	STATUS, RP1 ;GOTO BANK 0 GOTO BANK 2 RP1, RP0 -> "00".
		bcf 	STATUS, RP0 ;

		movf 	VALUE, W 	;MOVE WHATS IN "VALUE" INTO "W".

		bsf 	STATUS, RP1 ;GOTO BANK 2 GOTO BANK 2 RP1, RP0 -> "10".
		bcf 	STATUS, RP0 ;

		movwf 	EEDATA 		;WRITE THE CONTENTS OF "W" TO "EEDATA".

		bsf 	STATUS, RP1 ;GOTO BANK 3  RP1, RP0 -> "11"
		bsf 	STATUS, RP0 ;

		bcf 	EECON1, EEPGD ; POINT TO DATA MEMORY.
		bsf 	EECON1, WREN;ENABLE WRITING TO THE EEPROM.
							
		bcf 	INTCON, GIE ;DISABLE THE INTERRUPTS WHILE WE READ.
							
		movlw 	0x55 		;WRITE "H'55'" TO "W".
		movwf 	EECON2 		;MOVE WHATS IN "W" INTO EECON2.
		movlw 	0xAA 		;WRITE H"'AA'" TO "W".
		movwf	EECON2		;MOVE WHATS IN "W" INTO EECON2.

		bsf		EECON1,WR	;START THE WRITE OPERATION.
						
		btfsc	EECON1,WR	;WAIT FOR WRITE TO FINISH.
		goto	$-1			; GO BACK ONE LINE UNTIL FINISHED.

		bsf		INTCON,GIE	;RE-ENABLE THE INTERRUPTS.
							
		bcf		EECON1,WREN	;NOW DISABLE THE EEPROM "WRITE" SETTING.


		bcf 	STATUS, RP1 ;GOTO BANK 0 GOTO BANK 2 RP1, RP0 -> "00".
		bcf 	STATUS, RP0 ;

		return

;=============================================================================
;1.5 LOAD THE EEPROM VALUES INTO THE FCHAR_FREQ POSTIONS.
;
;PURPOSE: - The first thing to do is reset the EEPROM to "FF" and then after 
;			this is done we can load our new start frequency into it. We know
;			to do this because we have set a flag that says not only to 
;			display the frequency but also load it into the EEPROM. If this
;			flag is set then the program detours to this routine to load the
;			EERPOM as well as display the frequency.
;=============================================================================		
EEPROM_START_LOAD
		bcf		STATUS,IRP	;MAKING SURE THAT THE FIRST PAGE OF THE FSR
							; IS SELECTED.
		movlw	H'07'		;"7" THAT IS 0 - 7 OR EIGHT CHARACTERS.
		movwf	ADDR		;LOAD THIS INTO "ADDR".

		movlw	H'FF'		;SET "VALUE" TO "FF".
		movwf	VALUE

EE_FF_LOAD
		call	WRITE_EEPROM ; WE NEED "ADDR" AND "VALUE" SET BEFORE CALLING
							;  THIS ROUTINE.

		decf	ADDR		;"ADDR" IS THE LOCATION IN THE EERPOM. DECR IT BY
							; ONE.
		btfsc	STATUS,Z	;DECREMENT TO "0"?

		goto	EE_FREQ_SETUP ; NO. SO GO BACK UP AND LOAD "FF" INTO THAT POSITION.
		goto	EE_FF_LOAD	;YES. WE HAVE CLEARED THE EERPOM SO LETS NOW LOAD
							; THE NEW FREQUENCY.

EE_FREQ_SETUP
		movf	FCHAR_FREQ,W ; PUT THE START OF THE NEW FREQUENCY ADD INTO "W".
		movwf	FSR			;PUT IT INTO OUR INDIRECT REGISTER.

		movf	FCHAR_MAX,W	;LOAD OUR TOTAL NUMBER OF CHARS INTO "W".
		movwf	FCHAR_CNT	;NOW LOAD IT INTO A TEMP VARIABLE TO DECREMENT.

		movf	INDF,W		;LOAD OUR FIRST FREQ POSITION CHAR INTO "W"
		movwf	VALUE		;NOW MOVE IT INTO "VALUE" READY FOR THE WRITE TO
							;EERPOM.

		movlw	H'07'		;AGAIN THERE ARE EIGHT POSITIONS TO COPY OVER.
		movwf	ADDR

EE_FREQ_LOAD
		call	WRITE_EEPROM ; "ADDR" AND "VALUE" ARE SET SO LETS WRITE OUR FIRST
							;  POSITIONS VALUE.

		decf	FSR			;DECREMENT TO OUR SECOND POSITION.
		movf	INDF,W		;NOW MOVE THAT VALUE TO "W".
		movwf	VALUE		;...AND THEN  MOVE IT TO "VALUE READY FOR WRITE.

		decf	ADDR		;REDUCE OUR "ADDR" BY ONE.
		decf	FCHAR_CNT	;REDUCE OUR TEMP REGISTER BY ONE
		btfsc	STATUS,Z	;IS IT NOW "0"?
		goto	FINISH_EEPROM_LOAD ; YES. SO JUMP OUT.
		goto	EE_FREQ_LOAD;NO. SO WE DO IT ALL AGAIN WITH THE NEXT ADDRESS.

FINISH_EEPROM_LOAD
		bcf		EEPROM_FLAG,0 ;RESET THE FLAG THAT INDCATES NO START VALUE
							  ;HAS BEEN SAVED.		
		return

;=============================================================================
;2. 2 -ROUTINES - "DISPLAY FINAL FREQUENCY ON LCD" AND "ENTER FREQUENCY" LABEL.
;******************************************************************************
;
;PURPOSE: - There are two majour routines in this block.
;
;2.1 DISPLAY_FREQ_LCD
;2.2 LOAD_FREQ_LCD
;
;2.1 SETUP THE LCD TO DISPLAY THE "DISPLAY FREQUENCY" VIEW.
;
;PURPOSE: - All this routine does is display the "Hz" label, display the 
;			"MODE: DISPL FREQ" LABEL and then goto the routine DISP_CHAR
;			which displays the contents from the location loaded in "W".
;			In this case that position is the start of FREQ registers.
;
;			******************************************************************
;NOTE: - 	THIS ROUTINE IS WHERE WE JUMP FROM THE FIRST PART OF THE OVERALL
;			PROGRAM TO THE SECOND PART. THE PART WHERE WE CALCULATE THE "WORD".
;			******************************************************************

;=============================================================================	
DISPLAY_FREQ_LCD	
		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call	LCD_CLEAR	;CLEAR THE DISPLAY.

		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call 	HZ_LABEL	;"PRINT" THE LABEL "Hz" ON THE LCD.

		call	LCD_BUSY	;AGAIN MAKE SURE THE LCD IS NOT BUSY.
		call	DISPLAY_FREQ_LABEL ; NEXT LINE DOWN ON LCD DISPLAY THE MODE.

		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.

		movf	FCHAR_FREQ,W ; SET "W" TO THE START OF THE "FREQ" REGISTERS.		
		movwf	FSR			;  COPY THAT LOCATION TO THE INDIRECT REGISER.

		call	DISP_CHAR	;NOW CALL OUR DISPLAY CHARACTER ROUTINE.



;***********************************
;CALC THE WORD NEXT.....
;***********************************
		return

;=============================================================================
;2.2 SETUP LCD TO DISPLAY THE "LOAD FREQUENCY" VIEW.
;
;PURPOSE: - A simple routine that displays the "Hz" label and "ENTER FREQ"
;			label.
;=============================================================================	
LOAD_FREQ_LCD
		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call 	HZ_LABEL	;"PRINT" THE LABEL "Hz" ON THE LCD.

		call	LCD_BUSY	;AGAIN MAKE SURE THE LCD IS NOT BUSY.
		call	ENT_FREQ_LABEL; NEXT LINE DOWN ON LCD DISPLAY THE MODE.

		return

;=============================================================================
;2.2 SETUP LCD TO DISPLAY THE "LOAD FREQUENCY" VIEW.
;
;PURPOSE: - A simple routine that displays the "Hz" label and "ENTER FREQ"
;			label.
;=============================================================================	
LOAD_EE_FREQ_LCD
		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call 	HZ_LABEL	;"PRINT" THE LABEL "Hz" ON THE LCD.

		call	LCD_BUSY	;AGAIN MAKE SURE THE LCD IS NOT BUSY.
		call	ENT_EE_FREQ_LABEL; NEXT LINE DOWN ON LCD DISPLAY THE MODE.

		return


;=============================================================================
;3. ROUTINES FOR KEYPAD INTERRUPT START HERE.
;********************************************
;
;PURPOSE - There are 14 majour routines in this block.
;
;3.1 KEY_INTERRUPT	(Much like a case statement - diverts to other case stmts)
;3.2 FINISH_FREQ_ENTRY
;3.3 SAVE_OLD_FREQ
;3.4 SHIFT_KEYS
;3.5 CLEAR_POS
;3.6 LOAD_OLD_FREQ
;3.7 LOAD_NEW_FRQ
;3.8 STEP_1
;3.9 STEP_10
;3.10 STEP_100
;3.11 MENU
;3.12 RESET_STARTUP_FREQ
;3.13 CALIBRATE
;3.14 MENU_EXIT
;
;3.1 INTERRUPT AND ASSOCIATED ROUTINES FOR KEY ENTRY DESTINATION...
;
;PURPOSE: - We had an interrupt on the keyboard so lets go and get that char
;			that was entered and save it away and then first test to see if
;			it was higher than 1-9 and if it is see what was entered. By the 
;			way if the user has tried to enter another key other than 1-9 and
;			they are in the middle of entering in a new freq the system stops
;			them.
;
;			This is the central area where we fork off based on the key 
;			entered. All the routines in this group deal with the evaluation
;			of the key entered.
;
;			NOTE: All branching is by GOTO as we have a limited stack for
;				  using routines... 8 in all. So we land here from LOOP
;				  and depending on the key GOTO the routine for that entry.
;				  We always get here via a call so we need to have return.
;				  so... we branch with a GOTO here and have a return at the
;				  end of each routine.
;=============================================================================	
KEY_INTERRUPT
		movf	LAST_CHAR,W	;LOAD LAST_CHAR INTO "W".
		sublw	H'09'		;NO.
		btfsc	STATUS,C	;IS THE ENTERED KEY GREATER THAN 1-9?
		goto	ADD_NUMBER	;NO. SO WE NOW TEST TO SEE IF THIS IS THE FIRST
							; KEY ENTERED BY THE USER...
			
		goto	EVAL_OTHER_KEYS ; YES. THE KEY ENTERED IS NOT 1-9 BUT A
							;	  A KEY THAT MIGHT CLEAR IT, SAVE IT, SET A 
							;	  STEP OR GO TO THE MENU 
ADD_NUMBER
		btfss	NEW_FREQ_FLAG,INT_BIT ; IS THE FLAG FOR NEW FREQUENCY SET?
		call	SAVE_OLD_FREQ ; NO. SO WE NEED TO SAVE THE CURRENT FREQ AND 
							;	SET THIS FLAG.
			
		goto	SHIFT_KEYS	;THIS ROUTINE SHIFTS ALL ENTERED KEYS TO
							; ACCOMODATE THE NEW NUMBER ENTERED.

							;NOTE: - We now deal with those keypad keys 
							; "A", "B", "C" and "D". However if the user
							; is still in the middle of entering in a freq.
							; then we tell them they must finish the frequency
							; first before setting the STEP or accessing the
							; MENU.

EVAL_OTHER_KEYS
							;NOTE: - Clear entered contents or return back
							; to original frequency.

		movf	LAST_CHAR,W;MOVE OUR CHAR INTO "W".
		sublw	D'10'
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	CLEAR_POS 	;YES. THIS FIRST CLEARS AND IF ALREADY CLEAR
							; JUMPS BACK TO THE OLD FREQUENCY.

							;NOTE: - Accept entered frequency.

		movf	LAST_CHAR,W	;NO. MOVE OUR CHAR INTO "W".
		sublw	D'11'	
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	LOAD_NEW_FRQ;YES. EQUAL SO USER HAPPY WITH THIS NEW FREQ.

		btfss	NEW_FREQ_FLAG,INT_BIT ; NO. SO IS THE FLAG FOR NEW FREQUENCY SET?
		goto 	STEP_AND_MENU

		goto	FINISH_FREQ_ENTRY

STEP_AND_MENU
		movf	LAST_CHAR,W	;MOVE OUR CHAR INTO "W".
		sublw	D'12'	
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	STEP_1		;ENCODER MULT FOR STEPS OF "1".

		movf	LAST_CHAR,W	;MOVE OUR CHAR INTO "W".
		sublw	D'13'	
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	STEP_10		;ENCODER MULT FOR STEPS OF "10".
		
		movf	LAST_CHAR,W	;MOVE OUR CHAR INTO "W".
		sublw	D'14'	
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	STEP_100	;ENCODER MULT FOR STEPS OF "100".
		
		movf	LAST_CHAR,W	;MOVE OUR CHAR INTO "W".
		sublw	D'15'	
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	MENU		;GOTO OUR MENU FOR SUCH THINGS AS.....	


		return
CALL DEBUG_CHAR
CALL	WAIT_256MS

	
CALL DEBUGQ
CALL WAIT_256MS



;=============================================================================
;3.2 DISPLAY LABEL TELLING USER TO FINISH ENTERING IN FREQ FIRST...
;
;PURPOSE: - If the user tries to hit "A", "B", "C" OR "D" while the system is 
;			still in "ENTRY MODE" then we flash up a message saying so.
;			We then redisplay our "Hz" and "ENTER MODE" labels on the screen.
;			Because we are in the middle of entering in a frequency we need 
;			to also display the characters we have already entered. These are
;			located in the registers POS_1 - POS_10M.  Using the FSR or
;			indirect register we then set this to the start of the POS 
;			registers or POS_1 which is H'27'. We then finally call the 
;			DISP_CHAR routine which reverses the LCD write direction and 
;			display the currently entered characters.
;
;			Further we dont want to upset the apple cart in that we are in the
;			middle of entering a frequency so we dont reset ANY flags. The
;			NEW_FREQ_FLAG is already set when we come here...
;
;			Because we used GOTO to get here from FREQ_UPDATE which was called
;			from LOOP... we need a return.
;=============================================================================
FINISH_FREQ_ENTRY
		call	FINISH_FREQ_LCD ; CANT ACCESS THE MENU UNTIL THEY FINISH 
							;     FINISH ENTERING IN THE FREQ THEY STARTED ON.

		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call	LCD_CLEAR	;CLEAR THE DISPLAY.
	
		call	LOAD_FREQ_LCD

		movf	FCHAR_STRT,W;SET THE INDIRECT REGISTER TO THE LOWEST POS POSIT.
		movwf	FSR

		call	DISP_CHAR
	
		return				;(AND RETURN TO LOOP HERE)

;=============================================================================
;3.3 SAVE OLD FRFQUENCY AND THEN SET THE NEW FREQUENCY FLAG...
;
;PURPOSE: - The user has entered a character that is between 0-9 and its the
;			first char... so we save our char count from the old frequency
;			and set FCHAR_MAX to "0" ready for the user to enter in a new
;			frequency. The flag allows the program to stop the user from 
;			activating other routines not associated with the entry of
;			a new frequency.
;=============================================================================	
SAVE_OLD_FREQ
		movf	FCHAR_MAX,W	;LOAD OLD FCHAR_MAX INTO "W".
		movwf	FCHAR_MAX_TEMP ; SAVE FCHAR_MAX HERE.

		movlw	H'00'		;RESET FCHAR_MAX TO "0".
		movwf	FCHAR_MAX	

		bsf		NEW_FREQ_FLAG,	INT_BIT	; SET OUR NEW FREQUENCY FLAG.

							;NOTE - By setting this flag we are telling the
							; program next time we enter in a char that
							; we are in the middle of entering in a freq.


		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call	LCD_CLEAR	;NOW CLEAR THE LCD COMPLETELY READY FOR NEW FREQ.

		return				;(AND RETURN TO LOOP HERE)

;=============================================================================
;3.4 SHIFT EACH ENTERED KEY UP ONE IN THE REGISTERS..
;
;PURPOSE: - Before we can "print" the frequency entered onto the LCD they
;			need to be shifted up by one location. SHFT_KEYS is the routine
;			that moves each entered value up by one position in the registers
;			leaving the lowest postion vacant to copy in the new keyed number.	
;=============================================================================								
SHIFT_KEYS
		movf	FCHAR_MAX,W	;LOAD THE CURRENT NUMBER OF DIGITS ENTERED INTO "W".
		subwf	FCHAR_STRT,W;SUBTRACT "W" FROM THE FIRST LOCATION FOR FREQ.
		movwf	FSR			;FSR IS AT THE FIRST ADDRESS ENTERED.
		movf	FCHAR_MAX,W	;FCHAR_MAX IS OUR RUNNING COUNT.
		movwf	FCHAR_CNT	;WE CAN DEC THIS VAR WITHOUT SCREWING UP CHAR CNT.
		btfss	STATUS,Z	;CHECK FCHAR_CNT > 0?
		goto 	EXCEED_MAX_FREQ? 	;YES. WE HAVE MORE THAN ONE CHARACTER IN 
							;        OUR FREQ.. ELSE ITS THE FIRST CHAR...	
							
							;NOTE: - This next section deals with the 
							; situation where its the first character.

		movf	LAST_CHAR,W	;NO. THIS IS THE FIRST CHAR SO CHECK IF ITS >"0".
		btfsc	STATUS,Z	;CHECK FCHAR_CNT = 0?
		goto	ZERO_ERROR	;DROP OUT. CANT HAVE "0" AS FIRST CHAR.

							;NOTE: - We have a character greater than "0"
							; so simply add it to POS_1 via the FSR reg.

		incf	FCHAR_MAX	;YES ITS > "0" SO WE CAN NOW INC OUR COUNTERS.
		incf	FCHAR_CNT	;INCREMENT THE TEMPORARY COUNTER.
		movwf   INDF		;LOAD THE VALUE IN "W" INTO THE FSR REGISTER.

		call	LOAD_FREQ_LCD ; DISPLAY THE "Hz" LABEL AND "ENTER" LABEL.

							;NOTE: - Again we got to SHIFT_KEYS via a 
							; GOTO so we need to utilise the return at
							; at the end of DISP_CHAR as our return to
							; LOOP.

		goto 	DISP_CHAR	;GO TO THE CHAR DISPLAY ROUTINE. 
		
EXCEED_MAX_FREQ?
		movf	FCHAR_CNT,W	;NOW CHECK IF WE ARE AT X0Mhz AND CHECK X<6.
		sublw	D'06'
		btfsc	STATUS,C	;CHECK FCHAR_CNT > 7?
		goto	SHIFT_CHAR	;YES. ITS LOWER THAN "7" SO DONT CHECK 5MEG.
		movf	POS_1M,W	;NO. SO NOW CHECK IF OUR FREQ IS > "60MHz.
		sublw	D'05'
		btfsc	STATUS,C	;CHECK LAST_CHAR < 6?
		goto	SHIFT_CHAR	;YES ITS LOWER THAN "6" OR A POSSIBLE 60MHz.
		goto	FREQ_TO_BIG	;NO. DISPLAY THE "FREQ TO BIG" LABEL.

							;NOTE: - SHIFT_CHAR is central to this group
							; of routines in that it moves every current
							; entry up one register reveiling an empty
							; register at POS_1.
				
SHIFT_CHAR
		incf	FSR			;FSR IS ONE GREATER THAN OUR LAST CHAR... 	
		movf	INDF,W
		decf	FSR			;INCREMENT TO THE NEW LOCATION (ONE UP).	
		movwf	INDF		;DEPOSIT THE LOCATION VALUE THERE.
		incf	FSR			;SHIFT DOWN TO PREVIOUS LOCATION.		
		decf	FCHAR_CNT	;WE DIDNT "INC" THIS SO "DEC_ING" IT GOES BACK ONE.
		movf	FCHAR_CNT,F	;CHECK IF ITS BACK TO ZERO?
		btfsc	STATUS,Z	;IS THE DIGIT ENTERED COUNT ZERO?
		goto	LOAD_LAST_CHAR ;YES. ITS OUR LAST CHAR SO SAVE IT TO LOC="0"
		goto	SHIFT_CHAR	;NO. BACK TO THE TOP TO MOVE THE NEXT CHARACTER.

							;NOTE: - We have moved all the characters up one
							; so simply add in the key entered.
LOAD_LAST_CHAR
		incf	FCHAR_MAX	;INC OUR CHARACTER COUNT.
		movf	LAST_CHAR,W	;SAVE THE KEY TO THE LOCATION AND RETURN.
		movwf   INDF	
							
		goto	DISP_CHAR	;(RETURN TO LOOP FROM THIS ROUTINE.)

;=============================================================================
;3.5 CLEAR ENTERED CONTENTS OR RETURN BACK TO THE ORIGINAL FREQUENCY.
;
;PURPOSE: - The user can hit the "*"key twice... once to clear the contents and 
;			the second  time to bring back the frequency that was there
;			 beforehand. This routine does that.
;
;FLAGS CLEARED - We clear the INTERRUPT_FLAG. We also clear the NEW_FREQ_FLAG
;			but do this in the LOAD_OLD_FREQ routine called from here.
;=============================================================================	
CLEAR_POS
							;NOTE: - First we need to check if we are in the
							; middle of entering in a new freq. Because we
							; can hit the "*" key even when we are in the 
							; display mode we need to clear the interrupt
							; flag and simply drop out back to LOOP.

		btfsc	NEW_FREQ_FLAG,INT_BIT ; IS THE FLAG FOR NEW FREQUENCY SET?
		goto	CLEAR

		bcf		INTERRUPT_FLAG,	KEYS_BIT ; CLEAR OUR INTERRUPT FLAG FOR KEYS.

		return

							;NOTE: - OK the conditions here are we are in the 
							; middle of entering in a FREQ. The distingishing
							; test is whether FCHAR_MAX is greater than "0".
							; if it is make it "0". If it is already "0"
							; then reset flags. Again in this case there 
							; are two flags to be reset. NEW_FREQ_FLAG and
							; INTERRUPT_FLAG.

CLEAR
		movf	FCHAR_MAX,F	;COPY CHARACTERS BACK ONTO ITSELF.
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	LOAD_OLD_FREQ ; YES. THE CLEAR BUTTON HAS BEEN PRESSED WITH 
							;	NO CHARACTERS ENTERED. SO RESTORE OLD FREQ.	
							;   THE "NEW_FREQ_FLAG" IS SET IN THIS ROUTINE.
	
		movlw	H'00'
		movwf	FCHAR_MAX	;CLEAR OUR CHAR COUNTER.

		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call	LCD_CLEAR	;CLEAR THE DISPLAY.

		call	LOAD_FREQ_LCD ; DISPLAY OUR LOAD FREQ LABELS INCLUDING "Hz".

		bcf		INTERRUPT_FLAG,	KEYS_BIT ; CLEAR OUR INTERRUPT FLAG FOR KEYS.

		return				;(AND RETURN TO LOOP HERE)

;=============================================================================
;3.6 LOAD THE OLD FREQUENCY. (SHARED BY OTHER ROUTINES IN THIS BLOCK)
;
;PURPOSE: -	The user has decided they do not want to enter a new frequency 
; 			so return to the original frequency.
;=============================================================================	
LOAD_OLD_FREQ
		movf 	FCHAR_MAX_TEMP,W
		movwf	FCHAR_MAX

		call	DISPLAY_FREQ_LCD

		bcf		NEW_FREQ_FLAG,	INT_BIT	; CLEAR OUR EEPROM FLAG.

		return	;(AND RETURN TO LOOP HERE)

;=============================================================================
;3.7 LOAD THE ENTERED FREQUENCY TO THE VARIABLES THAT ARE USED FOR THE CALCULATION 
;    OF THE WORD FOR THE AD9851.
;
;PURPOSE: - Either we obtained a value from the EEPROM or the user has entered
;			one in. Its sitting in POS_1 TO POS_10M. We calculate the AD9851
;			WORD using FREQ_1 TO FREQ_10M so we need to copy the values in the
;			POS registers to the FREQ registers. This routine does that.
;=============================================================================	
LOAD_NEW_FRQ
		
		movf	FCHAR_MAX,F	;FIRST CHECK IF "FCHAR_MAX" HAS ANYTHING IN IT.
		btfsc	STATUS,Z	;IS "FCHAR_MAX" EQUAL TO "0"?
		goto	ZERO_ERROR	;YES. JUMP OUT AND INDICATE TO THE USER NO VALUE
							; ENTERED.

		movf	FCHAR_STRT,W;NOT = "0". SO SET OUR FIRST TEMP REG TO OUR FIRST
							;LOCATION.
		movwf	FCHAR_CNT

		movf	FCHAR_FREQ,W;SET OUR SECOND TEMP REGISTER TO THE "FREQ" LOC.
		movwf	FCHAR_CNT1

NEXT_POSITION
		movf	FCHAR_CNT,W	;MOVE OUR CURRENT LOCATION TO "W".
		movwf	FSR			;NOW SET THE INDIRECT REGISTER TO IT.

		movf	INDF,W		;COPY THE VALUE IN THAT REGISTER TO "W".
		movwf	VALUE		;THEN COPY THAT NUMBER TO "VALUE".
	
		movf	FCHAR_CNT1,W;DO THE SAME FOR THE SECOND REGISTER.
		movwf	FSR		

		movf	VALUE,W		;THIS TIME MOVE THE NUMBER IN "VALUE BACK TO "W".
		movwf	INDF		;AND THEN MOVE IT INTO OUR INDIRECT REGISTER.

		decf	FCHAR_CNT	;DECREMENT BY 1 BOTH OUR TEMP COUNTERS...
		decf	FCHAR_CNT1

		movf	FCHAR_CNT1,W;CHECK IF OUR "FREQ" COUNTER IS AT "0".
		sublw	H'28'		; DO THIS BY SUBBING THAT LOCATIN FROM IT.
		btfss	STATUS,Z	;IS THE RESULT "0"? (SKIP IF SET)
		goto	NEXT_POSITION ; NO. NOT SET SO DO THE WHOLE THING AGAIN.

		call	DISPLAY_FREQ_LCD ; YEP SET. SO DISPLAY IT ON THE LCD.

		btfsc	EEPROM_FLAG,0	;NOW... IS THIS FOR THE EEPROM?
		call	EEPROM_START_LOAD ; YES. SO LETS LOAD IT INTO THE EEPROM..


		bcf		NEW_FREQ_FLAG,	INT_BIT	; CLEAR OUR EEPROM FLAG.
		bcf		INTERRUPT_FLAG,	KEYS_BIT ; NO. SO SET THE INTERRUPT BIT.	

		return	;(AND RETURN TO LOOP HERE)

;=============================================================================
;3.8 CHANGE THE STEP VALUE FOR THE ENCODER TO 1Hz.
;
;PURPOSE: - To reset the step value we use in the encoder to "1"
;=============================================================================	
STEP_1
		movlw	H'01'
		movwf	STEP_VAL		

		call	STEP_CHANGED
		call	STEP_1_LCD

		call 	DISPLAY_FREQ_LCD

		bcf		NEW_FREQ_FLAG,	INT_BIT	; CLEAR OUR EEPROM FLAG.
		bcf		INTERRUPT_FLAG,	KEYS_BIT ; NO. SO SET THE INTERRUPT BIT.	
		
		return	;(AND RETURN TO LOOP HERE)


;=============================================================================
;3.9 CHANGE THE STEP VALUE FOR THE ENCODER TO 10Hz.
;
;PURPOSE: - To reset the step value we use in the encoder to "10"
;=============================================================================	
STEP_10

		movlw	H'0A'
		movwf	STEP_VAL		

		call	STEP_CHANGED
		call	STEP_10_LCD

		call 	DISPLAY_FREQ_LCD

		bcf		NEW_FREQ_FLAG,	INT_BIT	; CLEAR OUR EEPROM FLAG.
		bcf		INTERRUPT_FLAG,	KEYS_BIT ; NO. SO SET THE INTERRUPT BIT.	

		return	;(AND RETURN TO LOOP HERE)

;=============================================================================
;3.10 CHANGE THE STEP VALUE FOR THE ENCODER TO 100Hz.
;
;PURPOSE: - To reset the step value we use in the encoder to "100"
;=============================================================================	
STEP_100
		movlw	H'64'
		movwf	STEP_VAL		

		call	STEP_CHANGED
		call	STEP_100_LCD

		call 	DISPLAY_FREQ_LCD

		bcf		NEW_FREQ_FLAG,	INT_BIT	; CLEAR OUR EEPROM FLAG.
		bcf		INTERRUPT_FLAG,	KEYS_BIT ; NO. SO SET THE INTERRUPT BIT.	

		return	;(AND RETURN TO LOOP HERE)

;=============================================================================
;3.11 RUN THROUGH THE MENU OPTIONS.
;
;PURPOSE: -  If "D" has been entered then the user wants to go to the menu.
;			The menu is simply the "other" options displayed in a never
;			ending loop. The user can break the loop by pressing the "#" key.
;
;			This is a never ending loop that the user can break in on and 
;			select an option.  Anyone can add to this if they want to...
;							   *****************************************
;=============================================================================
MENU
		bcf		INTERRUPT_FLAG,	KEYS_BIT ; CLEAR OUR INTERRUPT FLAG FOR KEYS.
		bsf		MENU_FLAG, MENU_BIT	; SET THIS FLAG TO TEST FOR BEING IN MENU.

		call	 PRESS_#	;TELL USER TO PRESS # FOR ENTER...

						;NOTE - if "D" is pressed then the program goes into 
						; an indefinite loop for the user to select an option  
						; of exit...

		movlw	H'00'
		movwf	MENU_COUNT

MENU_LOOP
 		incf	MENU_COUNT

        call    RESET_STARTUP_FREQ_LCD ; GOT THE SET STARTUP FREQ ROUTINE.

        call    WAIT_500MS

		btfsc	INTERRUPT_FLAG,KEYS_BIT	; INTERRUPT FLAG BEEN SET?
		goto	MENU_INTERRUPT	;YES. SO GO FIND OUT WHAT KEY AND PROCESS....


		btfss	MENU_FLAG, MENU_BIT
		goto	EXIT_MENU_LOOP

		incf	MENU_COUNT
 
        call    CALIBRATE_LCD	;GOTO THE CALIBRATE ROUTINE.

        call    WAIT_500MS

		btfsc	INTERRUPT_FLAG,KEYS_BIT	; INTERRUPT FLAG BEEN SET?
		goto	MENU_INTERRUPT	;YES. SO GO FIND OUT WHAT KEY AND PROCESS....


		btfss	MENU_FLAG, MENU_BIT
		goto	EXIT_MENU_LOOP

		incf	MENU_COUNT

        call    MENU_EXIT_LCD	;EXIT

        call    WAIT_500MS
   

		btfsc	INTERRUPT_FLAG,KEYS_BIT	; INTERRUPT FLAG BEEN SET?
		goto	MENU_INTERRUPT	;YES. SO GO FIND OUT WHAT KEY AND PROCESS....


		btfss	MENU_FLAG, MENU_BIT
		goto	EXIT_MENU_LOOP

		movlw	H'00'
		movwf	MENU_COUNT

        goto    MENU_LOOP	; Repeat forever

EXIT_MENU_LOOP

		bcf		MENU_FLAG, MENU_BIT	; SET THIS FLAG TO TEST FOR BEING IN MENU.
		return

;=============================================================================
;
;=============================================================================
MENU_INTERRUPT

		movf	LAST_CHAR,W

		sublw	D'11'	
		btfsc	STATUS,Z	;IS THE ZERO BIT SET? DID THEY PRESS "#"?
		goto	TEST_FLAG	;YES. SO TEST THE FLAG.		
		goto	WRONG_KEY	;NO. SO CLEAR FLAGS AND RETURN.

;		call	DEBUGQ
;		CALL	WAIT_256MS
TEST_FLAG
		movf	MENU_COUNT,W	;MOVE OUR CHAR INTO "W".
		sublw	D'01'	
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	RESET_STARTUP_FREQ ; USER WANTS TO CHANGE EEPROM FREQ.

		movf	MENU_COUNT,W	;MOVE OUR CHAR INTO "W".
		sublw	D'02'	
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	CALIBRATE 	;USER WANTS TO CHANGE EEPROM FREQ.

		movf	MENU_COUNT,W	;MOVE OUR CHAR INTO "W".
		sublw	D'03'	
		btfsc	STATUS,Z	;IS THE ZERO BIT SET?
		goto	MENU_EXIT	; USER WANTS TO CHANGE EEPROM FREQ.

		call	DEBUGP
		CALL	WAIT_256MS
WRONG_KEY
		bcf		INTERRUPT_FLAG,	KEYS_BIT ;WRONG KEY PRESSED SO RESET
t	
		call	LCD_BUSY
		call	LCD_CLEAR						;			  THE INTERRUPT BIT.	
CALL DEBUG_CHAR
CALL WAIT_256MS			
goto t					;(RETURN TO LOOP FROM THIS ROUTINE.)




;=============================================================================
;3.12 USER WANTS TO ADD/CHANGE THE START UP FREQUENCY LOADED IN THE EEPROM.
;=============================================================================
RESET_STARTUP_FREQ
		bcf		MENU_FLAG, MENU_BIT	; CLR THIS FLAG TO TEST FOR BEING IN MENU.
		bcf		INTERRUPT_FLAG,	KEYS_BIT ; CLEAR OUR INTERRUPT FLAG FOR KEYS.

		call	LCD_BUSY	;MAKE SURE THE LCD IS NOT BUSY.
		call	LCD_CLEAR	;NOW CLEAR THE LCD COMPLETELY.

		bsf		EEPROM_FLAG,0 ;THIS TELLS THE SYSTEM TO SAVE TO EEPROM.

		call	LOAD_FREQ_LCD ; SETS UP THE LCD TO DISPLAY A LOAD PROMPT AND
							;	THE "Hz" LABEL.

		return				;(AND RETURN TO LOOP HERE)

;=============================================================================
;3.13 USER WANTS CALIBRATE THE FUNCTION GENERATOR.
;=============================================================================
CALIBRATE
		bcf		MENU_FLAG, MENU_BIT	; CLR THIS FLAG TO TEST FOR BEING IN MENU.

		return				;(AND RETURN TO LOOP HERE)

;=============================================================================
;3.14 USER WANTS EXIT THE MENU LOOP.
;=============================================================================
MENU_EXIT
		bcf		MENU_FLAG, MENU_BIT	; CLR THIS FLAG TO TEST FOR BEING IN MENU.

		return				;(AND RETURN TO LOOP HERE)

;=============================================================================
;GIVEN A STARTING POINT FOR FSR THIS WILL DISPLAY THE FREQUENCY...
;
;PURPOSE: - DISP_CHAR is a routine that not only displays each digit onto the 
;			LCD but it also determines the postion of the multiplier commas
;			- a maximum of two if the number entered is over 1,000,000....
;=============================================================================						
DISP_CHAR
		movlw	H'0D'		;POSITION CURSOR AND SWAPING THE LCD CHAR DIRECTION.
		call	LCD_LINE1_W	;FIRST LINE AT "W" CHARACTERS IN...
		call	LCD_DEC_CUR	;SWITCH THE DIRECTION OF WRITING ON THE LCD.

		movf	FCHAR_MAX,W	;MOVE THE CHAR COUNT TO "W".
		movwf	FCHAR_CNT	;SET NUMBER OF CHARACTERS TO HANDLE.

							;NOTE: - We now need to set both the "comma"
							; variables - the variables that hold the offset
							; against the char count for the position of the 
							; commas. This is done so that if the char count
							; is lower than "3" we dont inadvertently get
							; a comma on the LCD. Its reset to the real number
							; if required.
									
		movlw	H'09'		;MOVE "9" INTO "W".
		movwf	POS1		;SET THE VALUE TO "9".
		
		movlw	H'09'		;MOVE "9" INTO "W".		
		movwf	POS2		;SET THE VALUE TO "9".
				
							;NOTE: - A comma should appear every three digits.
							; So, we first determine if we have entered three
							; digits and if we have then insert a comma.
							; This also applies to the 6 digit postion as well.
							; Now we also need to know what position our
							; character count is at so we can compensate
							; for it as we count DOWN and not UP.
							; for example say we have entered 5 digits
							; well we redisplay 1,2,3 comma,4,5. 
							; The LCD will put in "1" first and then move to
							; the next postion. We switched the direction
							; so its moving right to left. Each time we 
							; decrement FCHAR_CNT. The decrementing goes
							; in the opposite direction. So when FCHAR_CNT
							; equals 2 we will have displayed 3 digits and
							; are ready for a comma... FCHAR_CNT now equals
							; POS1 and so we can call out for a comma....
							
							
		movf	FCHAR_CNT,W	;MOVE OUR TEMPORARY COUNT INTO "W".
		sublw	D'3'		;SUBTRACT "3" FROM IT.
		btfsc	STATUS,C	;IS OUR NUMBER BIGGER OR SMALLER THAN "3"?
		goto	LOAD_CHAR	;SMALLER, SO JUST GO AND PRINT A CHAR...
		movf	FCHAR_CNT,W	;BIGGER, SO WE NEED TO CALC POS1.
		movwf	POS1		;WE MOVE THE VALUE OF OUR CHAR COUNT TO POS1.	
		movlw	D'3'		;PUT 3 INTO "W".
		subwf	POS1		;SUB 3 FROM POS1 TO LEAVE THE DIFFERENCE.

		movf	FCHAR_CNT,W	;NOW FOR THE SECOND COMMA POSITION.
		sublw	D'6'		;SECOND COMMA POSITION HAPPENS AFTER 6 DIGITS.
		btfsc	STATUS,C	;IS OUR NUMBER BIGGER OR SMALLER THAN "6"?
		goto	LOAD_CHAR	;SMALLER, SO JUST GO AND PRINT A CHAR...
		movf	FCHAR_CNT,W	;BIGGER, SO WE NEED TO CALC POS2.
		movwf	POS2		;WE MOVE THE VALUE OF OUR CHAR COUNT TO POS2.
		movlw	D'6'		;PUT 6 INTO "W".
		subwf	POS2		;SUB 3 FROM POS1 TO LEAVE THE DIFFERENCE.
		
LOAD_CHAR
		movf	POS1,W		;PUT OUR "DIFFERENCE" INTO "W".
		subwf	FCHAR_CNT,W	;SUBTRACT OUR COUNT FROM "W" AND PUT IT IN "W".
		btfss	STATUS,Z	;IS IT EQUAL TO 0 CHECK THE ZERO STATUS REG...
		goto	COMMA_2		;NOT EQUAL SO NOW CHECK IF ITS = "6".
		movlw	','			;EQUAL SO PRINT THE COMMA ON THE LCD..
		call	LCD_DISP_CHAR ; THE COMMA IS IN "W". LETS PUT ON THE LCD. 		

COMMA_2
		movf	POS2,W		;PUT OUR "DIFFERENCE" INTO "W".
		subwf	FCHAR_CNT,W	;SUBTRACT OUR COUNT FROM "W" AND PUT IT IN "W".
		btfss	STATUS,Z	;IS IT EQUAL TO 0 CHECK THE ZERO STATUS REG...
		goto	ZERO		;NOT EQUAL CHECK IF WE DECREMENTED TO "0".
		movlw	','			;EQUAL SO PRINT THE COMMA ON THE LCD..
		call	LCD_DISP_CHAR ; THE COMMA IS IN "W". LETS PUT ON THE LCD. 		

ZERO
		movf	FCHAR_CNT, F;COPY OUR COUNTER TO ITSELF, THEN CHECK STATUS Z
		btfsc	STATUS, Z 	;CHECK IF THIS IS THE LAST CHARACTER HANDLED?
		goto	DISP_CLEAR	;YES SO NO MORE CHARS... EXIT OUT OF ROUTINE.....
				
CHAR_TO_LCD
		movf	INDF, W		;GET CHARACTER FROM POINTER POSITION.
		call	LCD_PUT_ASC ;SEND THIS CHAR TO ASCII CONVERSION ROUTINE
							; AND PRINT IT TO THE LCD.
		decf	FCHAR_CNT	;DECREMENT CHARACTER COUNTER.
		decf	FSR			;INCREMENT POINTER.
		goto	LOAD_CHAR	;REPEAT HAVING RESET OUR COUNTERS....	
			
DISP_CLEAR	
		call	LCD_INC_CUR	;CHANGE THE LCD WRITING DIRECTON TO LEFT/RIGHT.		
		
		bcf		INTERRUPT_FLAG,	KEYS_BIT ; CLEAR OUR INTERRUPT FLAG FOR KEYS.

		return				;(AND RETURN TO LOOP HERE)




;===========================================================================
;DELAY ROUTINES.
;***************
;
; DELAY_TIME	= ((DELAY_value * 3) + 4) * Cycle_time
; DELAY_VALUE	= (Delay_time - (4 * Cycle_time)) / (3 * Cycle_time)
;
; i.e. (@ 4MHz crystal)
; Delay_time	= ((32 * 3) + 4) * 1uSec
;		= 100uSec
; DELAY_value	= (500uSec - 4) / 3
;		= 165.33
;		= 165
;===========================================================================
WAIT_A_SEC	
		call WAIT_500MS
		call WAIT_500MS
		
WAIT_750MS	
		call WAIT_256MS
		call WAIT_256MS
		call WAIT_256MS
		
WAIT_500MS	
		call WAIT_128MS
		call WAIT_128MS
		call WAIT_128MS
		call WAIT_128MS


WAIT_256MS	
		call WAIT_128MS
		call WAIT_128MS
		
WAIT_128MS	
		movlw	D'240'		; 128 msec
		movwf	X_DLY_VAL
LOOP_128MS	
		call	DELAY		;WAIT 500uSec
		decfsz	X_DLY_VAL, F
		goto	LOOP_128MS
		return
		
WAIT_64MS	
		movlw	D'120'		; 64 msec
		movwf	X_DLY_VAL	;
LOOP_64MS	
		call	DELAY		;WAIT 500uSec
		decfsz	X_DLY_VAL, F
		goto	LOOP_64MS	
		return
		
WAIT_32MS	
		movlw	D'60'		; 30 msec
		movwF	X_DLY_VAL
LOOP_32MS	
		call	DELAY		; WAIT 500uSec
		decfsz	X_DLY_VAL, F
		goto	LOOP_32MS
		return


WAIT_15MS	
		movlw	D'30'		; 15 msec
		movwf	X_DLY_VAL
LOOP_15MS	
		call	DELAY		;wait 500uSec
		decfsz	X_DLY_VAL, F
		goto	LOOP_15MS
		return

;DELAY ROUTINE = 500uSec OR 1/2 A MILISECOND.
DELAY		
		movlw	H'FF'		; +1		1 CYCLE
		movwF	DELAY_VAL	; +2		1 CYCLE
		
DELAY_LOOP	
		decfsz	DELAY_VAL, F; step 1	1 CYCLE
		goto	DELAY_LOOP	; step 2	2 CYCLE
		return			; +3		2 CYCLE
		
		end
