;===============================================================================
;INCLUDE FILE: - KEYPAD_ROUTINES.INC
;
;PURPOSE: - GETKEY ROUTINE THAT RETURNS WITH THE KEY PRESSED IN W. 
;
; 1. GETKEY
; 
;PIN CONNECTIONS: -
;
;
;                     RB4        RB5         RB6          RB7
;                      |          |           |            |
;                      |          |           |            |
;                      |          |           |            |
;
;                    COL 1       COL 2       COL 3        COL 4   
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 1 |    1    |  |   2    |  |   3    |   |   A    |
;                 +---------+  +--------+  +--------+   +--------+ 
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 2 |    4    |  |   5    |  |   6    |   |   B    |
;                 +---------+  +--------+  +--------+   +--------+ 
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 3 |    7    |  |   8    |  |   9    |   |   C    |
;                 +---------+  +--------+  +--------+   +--------+  
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 4 |    *    |  |   0    |  |   #    |   |   D    |
;                 +---------+  +--------+  +--------+   +--------+  
;
;===============================================================================



;===============================================================================
; 1. GETKEY
;
;PURPOSE: - GETKEY ROUTINE THAT RETURNS WITH THE KEY PRESSED IN W. 
;(0 to 15, 16 = NO KEY) If any RB4-RB7 are high then the first thing this routine  
;does is identify which of the four keys is high.
;
;Its then sent off to one of COL1,2,3,4 where first D4 is set low and B (which
;was just high) is tested to see if its low. if it is we have found a number.
;===============================================================================
GETKEY
			btfss	PORTB, RB4
			goto	ROW1
			btfss	PORTB, RB5
			goto	ROW2
			btfss	PORTB, RB6
			goto	ROW3
			btfss	PORTB, RB7
			goto	ROW4
			retlw	D'16'
			
ROW1   		bsf		PORTD, RD4
			btfsc	PORTB, RB4
			retlw	D'1'		;1
			bsf		PORTD, RD5
			btfsc	PORTB, RB4
			retlw	D'4'		;4
			bsf		PORTD, RD6
			btfsc	PORTB, RB4
			retlw	D'7'		;7
			bsf		PORTD, RD7
			btfsc	PORTB, RB4
			retlw	D'10'		;*
			retlw	D'16'

ROW2        bsf		PORTD, RD4
			btfsc	PORTB, RB5
			retlw	D'2'		;2
			bsf		PORTD, RD5
			btfsc	PORTB, RB5
			retlw	D'5'		;5
			bsf		PORTD, RD6
			btfsc	PORTB, RB5
			retlw	D'8'		;8
			bsf		PORTD, RD7
			btfsc	PORTB, RB5
			retlw	D'0'		;0
			retlw	D'16'
			
ROW3        bsf		PORTD, RD4
			btfsc	PORTB, RB6
			retlw	D'3'		;3
			bsf		PORTD, RD5
			btfsc	PORTB, RB6
			retlw	D'6'		;6
			bsf		PORTD, RD6
			btfsc	PORTB, RB6
			retlw	D'9'		;9
			bsf		PORTD, RD7
			btfsc	PORTB, RB6
			retlw	D'11'		;#
			retlw	D'16'
			
ROW4        bsf		PORTD, RD4
			btfsc	PORTB, RB7
			retlw	D'12'		;A
			bsf		PORTD, RD5
			btfsc	PORTB, RB7
			retlw	D'13'		;B
			bsf		PORTD, RD6
			btfsc	PORTB, RB7
			retlw	D'14'		;C
			bsf		PORTD, RD7
			btfsc	PORTB, RB7
			retlw	D'15'		;D
			retlw	D'16'

;===============================================================================
; 1. GET_ENTER_KEY
;
;PURPOSE: - GETKEY ROUTINE THAT RETURNS WITH THE ENTER KEY PRESSED IN W. 
;(0 to 15, 16 = NO KEY) If any RB4-RB7 are high then the first thing this routine  
;does is identify which of the four keys is high.
;
;Its then sent off to one of COL1,2,3,4 where first D4 is set low and B (which
;was just high) is tested to see if its low. if it is we have found a number.
;===============================================================================
GET_ENTER_KEY
			btfss	PORTB, RB6
			goto	ROW3_1
			retlw	D'16'

ROW3_1
			btfsc	PORTB, RB6
			retlw	D'11'		;#